import json
import os
import webbrowser
from getpass import getuser
from json import loads
from logging import getLogger

from PyQt5.QtCore import QUrl, pyqtSignal
from PyQt5.QtWebEngineWidgets import QWebEngineView, QWebEngineProfile, QWebEnginePage
from PyQt5.QtWidgets import QDialog, QWidget, QVBoxLayout, QLabel, QPushButton, QStackedLayout, QLineEdit, QButtonGroup, \
    QRadioButton
from legendary.core import LegendaryCore

logger = getLogger("LoginWindow")


class LoginBrowser(QWebEngineView):
    def __init__(self):
        super(LoginBrowser, self).__init__()
        self.browser_profile = QWebEngineProfile("storage", self)
        self.webpage = QWebEnginePage(self.browser_profile, self)
        self.setPage(self.webpage)

    def createWindow(self, webengine_window_type):
        return self


class ImportWidget(QWidget):
    signal = pyqtSignal(bool)

    def __init__(self, core: LegendaryCore):

        super(ImportWidget, self).__init__()
        self.wine_paths = []
        self.layout = QVBoxLayout()
        self.core = core

        self.import_text = QLabel("<h2>Import from existing Epic Games Launcher installation</h2>\nYou will get "
                                  "logged out there")
        self.import_accept_button = QPushButton("Import")
        self.import_accept_button.clicked.connect(self.auth)
        appdata_paths = self.get_appdata_path()

        self.layout.addWidget(self.import_text)

        if len(appdata_paths) == 0:
            self.path = QLineEdit()
            self.path.setPlaceholderText("Path to Wineprefix (Not implemented)")
            self.layout.addWidget(self.path)
        else:
            self.btn_group = QButtonGroup()
        for i in appdata_paths:
            radio_button = QRadioButton(i)
            self.btn_group.addButton(radio_button)
            self.layout.addWidget(radio_button)

        # for i in appdata_paths:

        self.appdata_path_text = QLabel(f"Appdata path: {self.core.egl.appdata_path}")

        self.login_text = QLabel("")
        # self.layout.addWidget(self.btn_group)
        self.layout.addWidget(self.login_text)
        self.layout.addStretch(1)
        self.layout.addWidget(self.import_accept_button)

        self.setLayout(self.layout)

    def get_appdata_path(self) -> list:
        if self.core.egl.appdata_path:
            return [self.core.egl.appdata_path]

        else:  # Linux
            wine_paths = []
            possible_wine_paths = [os.path.expanduser('~/Games/epic-games-store/'),
                                   os.path.expanduser('~/.wine/')]

            for i in possible_wine_paths:
                if os.path.exists(i):
                    if os.path.exists(os.path.join(i, "drive_c/users", getuser(),
                                                   'Local Settings/Application Data/EpicGamesLauncher',
                                                   'Saved/Config/Windows')):
                        wine_paths.append(i)

            if len(wine_paths) > 0:
                appdata_dirs = [
                    os.path.join(i, "drive_c/users", getuser(), 'Local Settings/Application Data/EpicGamesLauncher',
                                 'Saved/Config/Windows') for i in wine_paths]
                return appdata_dirs
        return []

    def auth(self):
        self.import_accept_button.setDisabled(True)
        if not self.btn_group:
            self.core.egl.appdata_path = self.path.text()

        for i in self.btn_group.buttons():

            if i.isChecked():
                self.core.egl.appdata_path = i.text()
        try:
            if self.core.auth_import():
                logger.info(f"Logged in as {self.core.lgd.userdata['displayName']}")
                self.signal.emit(True)
                return

        except:
            pass

        self.import_accept_button.setDisabled(False)
        logger.warning("Error: No valid session found")
        self.login_text.setText("Error: No valid session found")


class SystemBrowserWidget(QWidget):
    signal = pyqtSignal(bool)

    def __init__(self, core: LegendaryCore):
        super(SystemBrowserWidget, self).__init__()
        self.core = core
        self.layout = QVBoxLayout()

        self.text = QLabel("Insert Sid from Browser")
        self.layout.addWidget(self.text)
        self.input = QLineEdit()
        self.input.setPlaceholderText("Insert sid from Browser")
        self.layout.addWidget(self.input)

        self.loading_text = QLabel("")
        self.layout.addWidget(self.loading_text)

        self.back_button = QPushButton("Back")
        self.back_button.clicked.connect(self.back)
        self.layout.addWidget(self.back_button)

        self.submit_button = QPushButton("Login")
        self.submit_button.clicked.connect(self.login)
        self.layout.addWidget(self.submit_button)

        self.setLayout(self.layout)

    def login(self):
        self.loading_text.setText("Loading")
        token = self.input.text()
        if token.startswith("{") and token.endswith("}"):
            token = json.loads(token)['sid']
        token = self.core.auth_sid(token)
        if self.core.auth_code(token):
            logger.info("Successfully logged in")
            self.signal.emit(True)
        else:
            logger.warning("Login failed")
            self.loading_text.setText("Login failed")

    def back(self):
        self.signal.emit(False)


class LoginWindow(QDialog):
    signal = pyqtSignal(bool)

    def __init__(self, core: LegendaryCore):
        super(LoginWindow, self).__init__()
        self.core = core
        self.success_code = False
        self.widget = QWidget()
        self.setGeometry(0, 0, 200, 300)
        self.welcome_layout = QVBoxLayout()
        self.title = QLabel(
            "<h2>Welcome to Rare the graphical interface for Legendary, an open source Epic Games alternative.</h2>\n<h3>Select one Option to Login</h3>")
        self.browser_btn = QPushButton("Use built in browser to login")
        self.browser_btn.clicked.connect(self.browser_login)
        self.browser_btn_normal = QPushButton("Use System browser")
        self.browser_btn_normal.clicked.connect(self.sys_browser_login)
        self.import_btn = QPushButton("Import from existing Epic Games installation")

        self.import_btn.clicked.connect(self.import_login)
        self.text = QLabel("")
        self.exit_btn = QPushButton("Exit App")
        self.exit_btn.clicked.connect(self.exit_login)

        self.welcome_layout.addWidget(self.title)
        self.welcome_layout.addWidget(self.browser_btn)
        self.welcome_layout.addWidget(self.browser_btn_normal)
        self.welcome_layout.addWidget(self.import_btn)
        self.welcome_layout.addWidget(self.text)
        self.welcome_layout.addWidget(self.exit_btn)
        self.widget.setLayout(self.welcome_layout)

        self.browser = LoginBrowser()
        self.browser.loadFinished.connect(self.check_for_sid_page)

        self.import_widget = ImportWidget(self.core)
        self.import_widget.signal.connect(self.login_signal)

        self.sys_browser_widget = SystemBrowserWidget(core)
        self.sys_browser_widget.signal.connect(self.login_signal)

        self.layout = QStackedLayout()
        self.layout.addWidget(self.widget)
        self.layout.addWidget(self.browser)
        self.layout.addWidget(self.import_widget)
        self.layout.addWidget(self.sys_browser_widget)
        self.setLayout(self.layout)
        self.show()

    def login_signal(self, b: bool):
        if b:
            self.success()
        else:
            self.layout.setCurrentIndex(0)
            self.text.setText("<h4 style='color: red'>Login failed</h4>")

    def login(self):
        self.exec_()
        return self.success_code

    def success(self):
        self.success_code = True
        self.close()

    def retry(self):
        self.__init__(self.core)

    def exit_login(self):
        self.code = 1
        self.close()

    def browser_login(self):
        self.setGeometry(0, 0, 800, 600)
        self.browser.load(QUrl(
            'https://www.epicgames.com/id/login?redirectUrl=https%3A%2F%2Fwww.epicgames.com%2Fid%2Fapi%2Fredirect'))
        self.layout.setCurrentIndex(1)

    def sys_browser_login(self):
        self.layout.setCurrentIndex(3)
        webbrowser.open(
            "https://www.epicgames.com/id/login?redirectUrl=https%3A%2F%2Fwww.epicgames.com%2Fid%2Fapi%2Fredirect")

    def import_login(self):
        self.layout.setCurrentIndex(2)

    def check_for_sid_page(self):
        if self.browser.url() == QUrl("https://www.epicgames.com/id/api/redirect"):
            self.browser.page().toPlainText(self.browser_auth)

    def browser_auth(self, json):
        token = self.core.auth_sid(loads(json)["sid"])
        if self.core.auth_code(token):
            logger.info(f"Successfully logged in as {self.core.lgd.userdata['displayName']}")
            self.success()
        else:
            self.layout.setCurrentIndex(0)
            logger.warning("Login failed")
            self.browser.close()
            self.text.setText("Login Failed")



import configparser
import os

config_path = os.path.expanduser("~") + "/.config/legendary/"
lgd_config = configparser.ConfigParser()
lgd_config.optionxform = str


def get_config() -> {}:
    lgd_config.read(config_path + "config.ini")
    lgd_config.optionxform = str
    return lgd_config


def set_config(new_config: {}):
    lgd_config = new_config
    lgd_config.write(open(config_path + "config.ini", "w"))



import json
import os
import string
from logging import getLogger

from PyQt5.QtCore import Qt
from PyQt5.QtWidgets import *
from legendary.core import LegendaryCore

from Rare.Tabs.GamesInstalled.GameWidget import GameWidget
from Rare.utils import legendaryUtils

logger = getLogger("InstalledList")


class GameListInstalled(QScrollArea):
    def __init__(self, core: LegendaryCore):
        super(GameListInstalled, self).__init__()

        self.core = core
        self.init_ui()

        # self.setLayout(self.layout)

    def init_ui(self):
        self.widget = QWidget()
        self.setVerticalScrollBarPolicy(Qt.ScrollBarAlwaysOn)
        self.setHorizontalScrollBarPolicy(Qt.ScrollBarAlwaysOff)

        self.layout = QVBoxLayout()

        self.update_button = QPushButton("Update")
        self.update_button.clicked.connect(self.update_list)
        self.layout.addWidget(self.update_button)
        self.widgets = {}
        games = sorted(self.core.get_installed_list(), key=lambda game: game.title)
        if games:
            for i in games:
                widget = GameWidget(i, self.core)
                widget.signal.connect(self.remove_game)
                self.widgets[i.app_name] = widget
                self.layout.addWidget(widget)

        else:
            not_installed_label = QLabel("No Games are installed. Do you want to import them?")
            not_installed_button = QPushButton("Import Games")
            not_installed_button.clicked.connect(self.import_games_prepare)
            self.layout.addWidget(not_installed_label)
            self.layout.addWidget(not_installed_button)

        self.widget.setLayout(self.layout)
        self.setWidget(self.widget)

    def remove_game(self, app_name: str):
        logger.info(f"Uninstall {app_name}")
        legendaryUtils.uninstall(app_name=app_name, core=self.core)
        self.widgets[app_name].setVisible(False)
        self.layout.removeWidget(self.widgets[app_name])
        self.widgets[app_name].deleteLater()
        self.widgets.pop(app_name)

    def import_games_prepare(self):
        # Automatically import from windows
        imported = 0
        if os.name == "nt":
            available_drives = ['%s:' % d for d in string.ascii_uppercase if os.path.exists('%s:' % d)]
            for drive in available_drives:
                path = f"{drive}/Program Files/Epic Games/"
                if os.path.exists(path):
                    imported += self.auto_import_games(path)

        else:
            possible_wineprefixes = [os.path.expanduser("~/.wine/"), os.path.expanduser("~/Games/epic-games-store/")]
            for wine_prefix in possible_wineprefixes:
                imported += self.auto_import_games(f"{wine_prefix}drive_c/Program Files/Epic Games/")
        if imported > 0:
            QMessageBox.information(self, "Imported Games", f"Successfully imported  {imported} Games")
            logger.info("Restarting app to import games")
        else:
            QMessageBox.information(self, "Imported Games", "No Games were found")

    def auto_import_games(self, game_path):
        imported = 0
        if not os.path.exists(game_path):
            return 0
        if os.listdir(game_path) == 0:
            logger.info(f"No Games found in {game_path}")
            return 0
        for path in os.listdir(game_path):
            json_path = game_path + path + "/.egstore"
            print(json_path)
            if not os.path.isdir(json_path):
                logger.info(f"Game at {game_path + path} doesn't exist")
                continue

            for file in os.listdir(json_path):
                if file.endswith(".mancpn"):
                    app_name = json.load(open(os.path.join(json_path, file)))["AppName"]
                    if legendaryUtils.import_game(self.core, app_name, game_path + path):
                        imported += 1
        return imported

    def update_list(self):
        self.core = LegendaryCore()
        del self.widget
        self.setWidget(QWidget())
        self.init_ui()
        self.update()



import logging
import sys

from PyQt5.QtWidgets import QApplication, QMessageBox
from legendary.core import LegendaryCore

from Rare import style_path
from Rare.MainWindow import MainWindow
from Rare.Start.Launch import LaunchDialog
from Rare.Start.Login import LoginWindow
from Rare.utils import RareConfig

logging.basicConfig(
    format='[%(name)s] %(levelname)s: %(message)s',
    level=logging.INFO
)
logger = logging.getLogger("Rare")
core = LegendaryCore()


def main():
    app = QApplication(sys.argv)
    if RareConfig.THEME == "default":
        app.setStyleSheet(open(style_path).read())
    # app.setStyleSheet(open("Styles/RareStyle.css").read())
    logger.info("Try if you are logged in")
    try:
        if core.login():

            logger.info("You are logged in")
        else:
            logger.error("Login Failed")
            main()

    except ValueError:
        logger.info("You ar not logged in. Open Login Window")
        login_window = LoginWindow(core)
        if not login_window.login():
            return
    except ConnectionError:
        QMessageBox.warning(app, "No Internet", "Connection Error, Failed to login. The offine mode is not implemented")
        # Start Offline mode
    launch_dialog = LaunchDialog(core)
    if RareConfig.THEME == "dark":
        launch_dialog.setStyleSheet(open(style_path).read())
    launch_dialog.exec_()
    mainwindow = MainWindow(core)
    app.exec_()


if __name__ == '__main__':
    main()



from PyQt5.QtCore import QThread, pyqtSignal
from PyQt5.QtWidgets import QDialog, QLabel, QProgressBar, QVBoxLayout
from legendary.core import LegendaryCore

from Rare.utils.RareUtils import download_images


class LaunchThread(QThread):
    download_progess = pyqtSignal(int)
    action = pyqtSignal(str)

    def __init__(self, core: LegendaryCore, parent=None):
        super(LaunchThread, self).__init__(parent)
        self.core = core

    def run(self):
        self.action.emit("Login")
        self.action.emit("Downloading Images")
        download_images(self.download_progess, self.core)
        self.action.emit("finish")


class LaunchDialog(QDialog):
    def __init__(self, core: LegendaryCore):
        super(LaunchDialog, self).__init__()

        self.title = QLabel("<h3>Launching Rare</h3>")
        self.thread = LaunchThread(core, self)
        self.thread.download_progess.connect(self.update_pb)
        self.thread.action.connect(self.info)
        self.info_pb = QProgressBar()
        self.info_pb.setMaximum(len(core.get_game_list()))
        self.info_text = QLabel("Logging in")
        self.layout = QVBoxLayout()

        self.layout.addWidget(self.title)
        self.layout.addWidget(self.info_pb)
        self.layout.addWidget(self.info_text)

        self.setLayout(self.layout)
        self.thread.start()

    def update_pb(self, i: int):
        self.info_pb.setValue(i)

    def info(self, text: str):
        if text == "finish":
            self.close()
        self.info_text.setText(text)



import os
import subprocess
from logging import getLogger

from PyQt5.QtCore import QThread, pyqtSignal, QProcess
from PyQt5.QtGui import QPixmap
from PyQt5.QtWidgets import QWidget, QLabel, QHBoxLayout, QVBoxLayout, QPushButton, QMessageBox

from Rare.Tabs.GamesUninstalled.InstallDialog import InstallDialog
from Rare.utils.RareConfig import IMAGE_DIR

logger = getLogger("Game")


class Thread(QThread):
    signal = pyqtSignal()

    def __init__(self, proc):
        super(Thread, self).__init__()
        self.proc: subprocess.Popen = proc

    def run(self):
        self.sleep(3)
        logger.info("Running ")
        while True:
            if not self.proc.poll():
                self.sleep(3)
            else:
                self.signal.emit()
                self.quit()
                logger.info("Kill")
                break


class UninstalledGameWidget(QWidget):
    finished = pyqtSignal()

    def __init__(self, game):
        super(UninstalledGameWidget, self).__init__()
        self.title = game.app_title
        self.app_name = game.app_name
        self.version = game.app_version
        self.layout = QHBoxLayout()
        self.game = game
        if os.path.exists(f"{IMAGE_DIR}/{game.app_name}/UninstalledArt.png"):
            pixmap = QPixmap(f"{IMAGE_DIR}/{game.app_name}/UninstalledArt.png")
            pixmap = pixmap.scaled(120, 160)
            self.image = QLabel()
            self.image.setPixmap(pixmap)

        self.child_layout = QVBoxLayout()

        self.title_label = QLabel(f"<h2>{self.title}</h2>")
        self.app_name_label = QLabel(f"App Name: {self.app_name}")
        self.version_label = QLabel(f"Version: {self.version}")
        self.install_button = QPushButton("Install")
        self.install_button.setFixedWidth(120)
        self.install_button.clicked.connect(self.install)

        self.child_layout.addWidget(self.title_label)
        self.child_layout.addWidget(self.app_name_label)
        self.child_layout.addWidget(self.version_label)
        self.child_layout.addWidget(self.install_button)
        self.child_layout.addStretch(1)
        self.layout.addWidget(self.image)
        self.layout.addLayout(self.child_layout)

        self.layout.addStretch(1)
        self.setLayout(self.layout)

    def install(self):
        logger.info("install " + self.title)
        dia = InstallDialog(self.game)
        data = dia.get_data()
        print(data)
        if data != 0:
            path = data.get("install_path")
            logger.info(f"install {self.app_name} in path {path}")
            # TODO
            self.install_button.setDisabled(True)
            self.install_button.setText("installing")
            self.proc = QProcess()
            self.proc.start("legendary", ["-y", "install", self.app_name, "--base-path", path])
            self.proc.finished.connect(self.download_finished)

        else:
            logger.info("Download canceled")

    def download_finished(self, code):
        self.setVisible(False)
        logger.info(f"Download finished. code: {code}")
        QMessageBox.information(self, "Download finished", "Download has finished")
        self.finished.emit()



from logging import getLogger

from PyQt5.QtCore import Qt
from PyQt5.QtWidgets import QVBoxLayout, QLabel, QPushButton, QScrollArea, QWidget
from legendary.core import LegendaryCore

from Rare.Tabs.Settings.RareSettingsForm import RareSettingsForm
from Rare.ext.SettingsForm import SettingsForm
from Rare.utils.constants import default_settings, legendary_settings
from Rare.utils.legendaryUtils import get_name

logger = getLogger("Settings")


class SettingsTab(QScrollArea):
    def __init__(self, core: LegendaryCore):
        super(SettingsTab, self).__init__()
        self.core = core
        self.widget = QWidget()
        self.setVerticalScrollBarPolicy(Qt.ScrollBarAsNeeded)
        self.setHorizontalScrollBarPolicy(Qt.ScrollBarAsNeeded)
        # Settings
        self.layout = QVBoxLayout()
        self.layout.addWidget(QLabel("<h1>Rare Settings</h1>"))
        self.logged_in_as = QLabel(f"Logged in as {get_name()}")
        # self.legendary_form =SettingsForm()

        self.default_settings = SettingsForm("default", default_settings, True)
        self.legendary_form = SettingsForm("Legendary", legendary_settings, False)

        self.rare_form = RareSettingsForm()

        self.logout_button = QPushButton("Logout")
        self.logout_button.clicked.connect(self.logout)
        self.layout.addWidget(self.logged_in_as)

        self.layout.addWidget(self.rare_form)
        self.layout.addWidget(self.default_settings)
        self.layout.addWidget(self.legendary_form)

        self.layout.addStretch(1)
        self.layout.addWidget(self.logout_button)

        self.info_label = QLabel("<h2>Credits</h2>")
        self.infotext = QLabel("Developers : Dummerle, CommandMC\nLegendary Dev: Rodney\nLicense: GPL v.3")

        self.layout.addWidget(self.info_label)
        self.layout.addWidget(self.infotext)

        self.widget.setLayout(self.layout)
        self.widget.setFixedWidth(self.width())
        self.setWidget(self.widget)

    def logout(self):
        self.core.lgd.invalidate_userdata()
        exit(0)



import os

from PyQt5.QtWidgets import QLineEdit, QPushButton, QLabel, QVBoxLayout, QDialog, QHBoxLayout


class InstallDialog(QDialog):
    def __init__(self, game):
        super(InstallDialog, self).__init__()
        self.setWindowTitle("Install Game")
        self.layout = QVBoxLayout()
        self.yes = False
        self.install_path = QLineEdit(f"{os.path.expanduser('~')}/legendary")
        self.options = QLabel("Some Settings")
        # self.layout.addWidget(self.options)

        self.layout.addStretch(1)
        self.yes_button = QPushButton("Install")
        self.yes_button.clicked.connect(self.close)
        self.cancel_button = QPushButton("cancel")

        self.layout.addWidget(self.options)
        self.layout.addWidget(self.install_path)

        self.button_layout = QHBoxLayout()
        self.button_layout.addWidget(self.yes_button)
        self.button_layout.addWidget(self.cancel_button)
        self.yes_button.clicked.connect(self.accept)
        self.cancel_button.clicked.connect(self.cancel)
        self.layout.addLayout(self.button_layout)
        self.setLayout(self.layout)

    def get_data(self) -> dict:
        self.exec_()
        return {
            "install_path": self.install_path.text()
        } if self.yes else 0

    def accept(self):
        self.yes = True
        self.close()

    def cancel(self):
        self.yes = False
        self.close()



import os

from PyQt5.QtCore import Qt
from PyQt5.QtWidgets import *


class CustomQLabel(QLabel):
    def __init__(self, *__args):
        super().__init__(*__args)
        self.app_name_label.setTextInteractionFlags(Qt.TextSelectableByMouse)


class PathEdit(QWidget):
    def __init__(self, text: str = "", type_of_file: QFileDialog.FileType = QFileDialog.AnyFile, infotext: str = "",
                 filter: str = None):
        super(PathEdit, self).__init__()
        self.filter = filter
        self.type_of_file = type_of_file
        self.info_text = infotext
        self.layout = QHBoxLayout()
        self.text_edit = QLineEdit(text)
        self.path_select = QPushButton("Select Path")
        self.path_select.clicked.connect(self.set_path)
        self.layout.addWidget(self.text_edit)
        self.layout.addWidget(self.path_select)

        self.setLayout(self.layout)

    def setPlaceholderText(self, text: str):
        self.text_edit.setPlaceholderText(text)

    def text(self):
        return self.text_edit.text()

    def set_path(self):
        dlg = QFileDialog(self, "Choose Path", os.path.expanduser("~/"))
        dlg.setFileMode(self.type_of_file)
        if self.filter:
            dlg.setFilter(self.filter)
        if dlg.exec_():
            names = dlg.selectedFiles()
            self.text_edit.setText(names[0])



from PyQt5.QtCore import QUrl
from PyQt5.QtWebEngineWidgets import QWebEngineView, QWebEngineProfile, QWebEnginePage


class BrowserTab(QWebEngineView):
    def __init__(self):
        super(BrowserTab, self).__init__()
        self.profile = QWebEngineProfile("storage", self)
        self.webpage = QWebEnginePage(self.profile, self)
        self.webpage.javaScriptConsoleMessage = lambda level, msg, line, sourceID: None
        self.setPage(self.webpage)
        self.load(QUrl("https://www.epicgames.com/store/"))
        self.show()

    def createWindow(self, QWebEnginePage_WebWindowType):
        return self



import json
import os
from logging import getLogger

from PyQt5.QtWidgets import *
from legendary.core import LegendaryCore

from Rare.utils import legendaryUtils

logger = getLogger("ImportDialog")


class ImportDialog(QDialog):
    def __init__(self, core: LegendaryCore):
        super(ImportDialog, self).__init__()
        self.core = core
        self.title_text = QLabel("Select Path")
        self.path_field = QLineEdit()
        self.info_text = QLabel("")
        self.import_button = QPushButton("Import")
        self.import_button.clicked.connect(self.button_click)

        self.layout = QVBoxLayout()
        self.layout.addWidget(self.title_text)
        self.layout.addWidget(self.path_field)
        self.layout.addWidget(self.info_text)
        self.layout.addWidget(self.import_button)

        self.setLayout(self.layout)

    def button_click(self):
        if self.import_game():
            QMessageBox.about(self, "Info", "Please restart App to reload installed Games")
            self.close()
        else:
            self.info_text.setText("Failed to import Game")

    def import_game(self):
        path = self.path_field.text()
        print()
        if not path.endswith("/"):
            path = path + "/"
        file = ""

        for i in os.listdir(os.path.join(path, ".egstore")):
            if i.endswith(".mancpn"):
                file = path + ".egstore/" + i
                break
        else:
            logger.warning("File was not found")
            return False
        app_name = json.load(open(file, "r"))["AppName"]
        if legendaryUtils.import_game(self.core, app_name=app_name, path=path):
            return True
        else:
            logger.warning("Failed to import")
            return False

    def import_dialog(self):
        self.exec_()



import logging
import os
import subprocess

from PyQt5.QtCore import QProcess, QProcessEnvironment
from legendary.core import LegendaryCore

logger = logging.getLogger("LGD")
core = LegendaryCore()


def get_installed():
    return sorted(core.get_installed_list(), key=lambda name: name.title)


def get_installed_names():
    return [i.app_name for i in core.get_installed_list()]


def get_not_installed():
    games = []
    installed = get_installed_names()
    for game in get_games():
        if not game.app_name in installed:
            games.append(game)
    return games


# return (games, dlcs)
def get_games_and_dlcs():
    if not core.login():
        print("Login Failed")
        exit(1)
    return core.get_game_and_dlc_list()


def get_game_by_name(name: str):
    return core.get_game(name)


def get_games():
    if not core.login():
        print("Login Failed")
        return None

    return sorted(core.get_game_list(), key=lambda x: x.app_title)


def get_games_sorted():
    if not core.login():
        logging.error("No login")


def launch_game(app_name: str, lgd_core: LegendaryCore, offline: bool = False, skip_version_check: bool = False,
                username_override=None,
                wine_bin: str = None, wine_prfix: str = None, language: str = None, wrapper=None,
                no_wine: bool = os.name == "nt", extra: [] = None):
    game = lgd_core.get_installed_game(app_name)
    if not game:
        print("Game not found")
        return None
    if game.is_dlc:
        print("Game is dlc")
        return None
    if not os.path.exists(game.install_path):
        print("Game doesn't exist")
        return None

    if not offline:
        print("logging in")
        if not lgd_core.login():
            return None
        if not skip_version_check and not core.is_noupdate_game(app_name):
            # check updates
            try:
                latest = lgd_core.get_asset(app_name, update=True)
            except ValueError:
                print("Metadata doesn't exist")
                return None
            if latest.build_version != game.version:
                print("Please update game")
                return None
    params, cwd, env = lgd_core.get_launch_parameters(app_name=app_name, offline=offline,
                                                      extra_args=extra, user=username_override,
                                                      wine_bin=wine_bin, wine_pfx=wine_prfix,
                                                      language=language, wrapper=wrapper,
                                                      disable_wine=no_wine)
    process = QProcess()
    process.setWorkingDirectory(cwd)
    environment = QProcessEnvironment()
    for e in env:
        environment.insert(e, env[e])
    process.setProcessEnvironment(environment)
    process.start(params[0], params[1:])
    return process


def get_updates():
    update_games = []
    for game in core.get_installed_list():
        update_games.append(game) if get_game_by_name(game.app_name).app_version != game.version else None
    return update_games


def logout():
    core.lgd.invalidate_userdata()


def install(app_name: str, path: str = None):
    subprocess.Popen(f"legendary -y install {app_name}".split(" "))
    # TODO


def login(sid):
    code = core.auth_sid(sid)
    if code != '':
        return core.auth_code(code)
    else:
        return False


def get_name():
    return core.lgd.userdata["displayName"]


def uninstall(app_name: str, core):
    igame = core.get_installed_game(app_name)
    try:
        # Remove DLC first so directory is empty when game uninstall runs
        dlcs = core.get_dlc_for_game(app_name)
        for dlc in dlcs:
            if (idlc := core.get_installed_game(dlc.app_name)) is not None:
                logger.info(f'Uninstalling DLC "{dlc.app_name}"...')
                core.uninstall_game(idlc, delete_files=True)

        logger.info(f'Removing "{igame.title}" from "{igame.install_path}"...')
        core.uninstall_game(igame, delete_files=True, delete_root_directory=True)
        logger.info('Game has been uninstalled.')

    except Exception as e:
        logger.warning(f'Removing game failed: {e!r}, please remove {igame.install_path} manually.')


def import_game(core: LegendaryCore, app_name: str, path: str):
    logger.info("Import " + app_name)
    igame = core.get_game(app_name)
    manifest, jgame = core.import_game(igame, path)
    exe_path = os.path.join(path, manifest.meta.launch_exe.lstrip('/'))
    total = len(manifest.file_manifest_list.elements)
    found = sum(os.path.exists(os.path.join(path, f.filename))
                for f in manifest.file_manifest_list.elements)
    ratio = found / total
    if not os.path.exists(exe_path):
        logger.error(f"Game {igame.app_title} failed to import")
        return False
    if ratio < 0.95:
        logger.error(
            "Game files are missing. It may be not the lates version ore it is corrupt")
        return False
    core.install_game(jgame)
    if jgame.needs_verification:
        logger.info(logger.info(
            f'NOTE: The game installation will have to be verified before it can be updated '
            f'with legendary. Run "legendary repair {app_name}" to do so.'))

    logger.info("Successfully imported Game: " + igame.app_title)
    return True



from PyQt5.QtWidgets import QMainWindow, QTabWidget, QWidget

from Rare.Tabs import SettingsTab, UpdateTab, GameListInstalled, GameListUninstalled, BrowserTab


class MainWindow(QMainWindow):

    def __init__(self, core):
        super().__init__()
        self.setWindowTitle("Rare - GUI for legendary-gl")
        self.setGeometry(0, 0, 1200, 900)
        self.setCentralWidget(TabWidget(core))
        self.show()


class TabWidget(QTabWidget):

    def __init__(self, core):
        super(QWidget, self).__init__()
        self.game_list = GameListInstalled(core)
        self.addTab(self.game_list, "Games")
        self.uninstalled_games = GameListUninstalled(core)
        self.uninstalled_games.finished.connect(lambda: self.game_list.update_list())
        self.addTab(self.uninstalled_games, "Install Games")

        self.update_tab = UpdateTab(core)
        self.addTab(self.update_tab, "Updates")

        self.browser = BrowserTab()
        self.addTab(self.browser, "Store")

        self.settings = SettingsTab(core)
        self.addTab(self.settings, "Settings")


import configparser
import os
from logging import getLogger

config_path = os.path.join(os.path.expanduser("~"), ".config/Rare/")
rare_config = configparser.ConfigParser()

logger = getLogger("Config")
rare_config.read(config_path + "config.ini")

if not os.path.exists(config_path):
    os.makedirs(config_path)
    rare_config["Rare"] = {
        "IMAGE_DIR": os.path.expanduser("~/.cache/images"),
        "theme": "default"
    }
    rare_config.write(open(config_path + "config.ini", "w"))
elif not rare_config.sections():
    rare_config["Rare"] = {
        "IMAGE_DIR": os.path.expanduser("~/.cache/images"),
        "theme": "default"
    }
    rare_config.write(open(config_path + "config.ini", "w"))


def get_config() -> {}:
    return rare_config.__dict__["_sections"]


def set_config(new_config: {}):
    rare_config.__dict__["_sections"] = new_config
    rare_config.write(open(config_path + "config.ini", "w"))


IMAGE_DIR = rare_config["Rare"]["IMAGE_DIR"]
THEME = rare_config["Rare"]["theme"]



import json
import os
import sys
from logging import getLogger

import requests
from PIL import Image
from PyQt5.QtCore import pyqtSignal
from legendary.core import LegendaryCore

from Rare.utils.RareConfig import IMAGE_DIR

logger = getLogger("Utils")


def download_images(signal: pyqtSignal, core: LegendaryCore):
    if not os.path.isdir(IMAGE_DIR):
        os.makedirs(IMAGE_DIR)
        logger.info("Create Image dir")

    # Download Images
    for i, game in enumerate(sorted(core.get_game_list(), key=lambda x: x.app_title)):

        if not os.path.isdir(f"{IMAGE_DIR}/" + game.app_name):
            os.mkdir(f"{IMAGE_DIR}/" + game.app_name)

        if not os.path.isfile(f"{IMAGE_DIR}/{game.app_name}/image.json"):
            json_data = {"DieselGameBoxTall": None, "DieselGameBoxLogo": None}
        else:
            json_data = json.load(open(f"{IMAGE_DIR}/{game.app_name}/image.json", "r"))

        for image in game.metadata["keyImages"]:
            if image["type"] == "DieselGameBoxTall" or image["type"] == "DieselGameBoxLogo":

                if json_data[image["type"]] != image["md5"] or not os.path.isfile(
                        f"{IMAGE_DIR}/{game.app_name}/{image['type']}.png"):
                    # Download
                    json_data[image["type"]] = image["md5"]
                    # os.remove(f"{IMAGE_DIR}/{game.app_name}/{image['type']}.png")
                    json.dump(json_data, open(f"{IMAGE_DIR}/{game.app_name}/image.json", "w"))
                    logger.info(f"Download Image for Game: {game.app_title}")
                    url = image["url"]
                    with open(f"{IMAGE_DIR}/{game.app_name}/{image['type']}.png", "wb") as f:
                        f.write(requests.get(url).content)
                        f.close()

        if not os.path.isfile(f'{IMAGE_DIR}/' + game.app_name + '/UninstalledArt.png'):

            if os.path.isfile(f'{IMAGE_DIR}/' + game.app_name + '/DieselGameBoxTall.png'):
                # finalArt = Image.open(f'{IMAGE_DIR}/' + game.app_name + '/DieselGameBoxTall.png')
                # finalArt.save(f'{IMAGE_DIR}/{game.app_name}/FinalArt.png')
                # And same with the grayscale one

                bg = Image.open(f"{IMAGE_DIR}/{game.app_name}/DieselGameBoxTall.png")
                uninstalledArt = bg.convert('L')
                uninstalledArt.save(f'{IMAGE_DIR}/{game.app_name}/UninstalledArt.png')
            elif os.path.isfile(f"{IMAGE_DIR}/{game.app_name}/DieselGameBoxLogo.png"):
                bg: Image.Image = Image.open(f"{IMAGE_DIR}/{game.app_name}/DieselGameBoxLogo.png")
                bg = bg.resize((int(bg.size[1] * 3 / 4), bg.size[1]))
                logo = Image.open(f'{IMAGE_DIR}/{game.app_name}/DieselGameBoxLogo.png').convert('RGBA')
                wpercent = ((bg.size[0] * (3 / 4)) / float(logo.size[0]))
                hsize = int((float(logo.size[1]) * float(wpercent)))
                logo = logo.resize((int(bg.size[0] * (3 / 4)), hsize), Image.ANTIALIAS)
                # Calculate where the image has to be placed
                pasteX = int((bg.size[0] - logo.size[0]) / 2)
                pasteY = int((bg.size[1] - logo.size[1]) / 2)
                # And finally copy the background and paste in the image
                # finalArt = bg.copy()
                # finalArt.paste(logo, (pasteX, pasteY), logo)
                # Write out the file
                # finalArt.save(f'{IMAGE_DIR}/' + game.app_name + '/FinalArt.png')
                logoCopy = logo.copy()
                logoCopy.putalpha(int(256 * 3 / 4))
                logo.paste(logoCopy, logo)
                uninstalledArt = bg.copy()
                uninstalledArt.paste(logo, (pasteX, pasteY), logo)
                uninstalledArt = uninstalledArt.convert('L')
                uninstalledArt.save(f'{IMAGE_DIR}/' + game.app_name + '/UninstalledArt.png')
            else:
                logger.warning(f"File {IMAGE_DIR}/{game.app_name}/DieselGameBoxTall.png dowsn't exist")
        signal.emit(i)


def restart():
    os.execv(sys.executable, ['python'] + sys.argv)



from PyQt5.QtCore import Qt, pyqtSignal
from PyQt5.QtWidgets import *
from legendary.core import LegendaryCore

from Rare.Tabs.GamesUninstalled.GameWidget import UninstalledGameWidget
from Rare.utils.Dialogs.ImportDialog import ImportDialog


class GameListUninstalled(QScrollArea):
    finished = pyqtSignal()

    def __init__(self, core: LegendaryCore):
        super(GameListUninstalled, self).__init__()
        self.core = core
        self.widget = QWidget()
        self.setVerticalScrollBarPolicy(Qt.ScrollBarAsNeeded)
        self.setHorizontalScrollBarPolicy(Qt.ScrollBarAsNeeded)

        self.layout = QVBoxLayout()

        self.filter = QLineEdit()
        self.filter.textChanged.connect(self.filter_games)
        self.filter.setPlaceholderText("Filter Games")
        self.import_button = QPushButton("Import installed Game from Epic Games Store")
        self.import_button.clicked.connect(self.import_game)
        self.layout.addWidget(self.filter)
        self.layout.addWidget(self.import_button)
        self.widgets_uninstalled = []
        games = []
        installed = [i.app_name for i in core.get_installed_list()]
        for game in core.get_game_list():
            if not game.app_name in installed:
                games.append(game)
        games = sorted(games, key=lambda x: x.app_title)
        for game in games:
            game_widget = UninstalledGameWidget(game)
            game_widget.finished.connect(lambda: self.finished.emit())
            self.layout.addWidget(game_widget)
            self.widgets_uninstalled.append(game_widget)

        self.layout.addStretch(1)
        self.widget.setLayout(self.layout)
        self.setWidget(self.widget)

    def filter_games(self):
        for i in self.widgets_uninstalled:
            if self.filter.text().lower() in i.game.app_title.lower() + i.game.app_name.lower():
                i.setVisible(True)
            else:
                i.setVisible(False)

    def import_game(self):
        import_dia = ImportDialog(self.core)
        import_dia.import_dialog()



# (Info Text, Type of input, legendary conf name, flags)
# possible flags: wine, path, only_int, binary(ComboBox), Combobox content

default_settings = [
    ("Wine Prefix", "QLineEdit", "wine_prefix", ["wine", "path"]),
    ("Wine Executable", "QLineEdit", "wine_executable", ["wine"])
]

legendary_settings = [

    ("Locale", "QLineEdit", "locale"),
    ("Max Workers", "QLineEdit", "max_workers", ["only_int"]),
    ("Default install dir", "QLineEdit", "install_dir", ["path"]),
    ("Max Memory", "QLineEdit", "max_memory", ["only_int"]),
    ("EGL Sync", "QComboBox", "egl_sync", ["binary"]),
    ("No Wine", "QComboBox", "no_wine", ["binary"])
]

game_settings = default_settings + [
    ("Offline", "QComboBox", "offline", ["binary"]),
    ("Skip Update Check", "QComboBox", "skip_update_check", ["binary"]),
    ("Launch Parameter", "start_params", "QLineEdit"),
    ("Wrapper", "QLineEdit", "wrapper"),
    ("No Wine", "QComboBox", "no_wine", ["binary", "wine"])
]



import configparser
import os
from logging import getLogger

from PyQt5 import QtWidgets, QtCore
from PyQt5.QtGui import QIntValidator
from PyQt5.QtWidgets import *

from Rare.ext.QtExtensions import PathEdit
from Rare.utils import legendaryConfig

logger = getLogger("SettingsForm")


class SettingsForm(QGroupBox):

    def __init__(self, app_name, settings: [tuple], table: bool):
        super(SettingsForm, self).__init__()
        self.config = legendaryConfig.get_config()
        self.app_name = app_name
        self.form = QFormLayout()
        self.rows = []
        self.has_table = table
        for i in settings:
            self.add_Row(*i)
        if table:

            if not f"{self.app_name}.env" in self.config:
                env_vars = {}
            else:
                env_vars = self.config[f"{self.app_name}.env"]

            if env_vars:
                self.table = QTableWidget(len(env_vars), 2)
                header = self.table.horizontalHeader()
                header.setSectionResizeMode(0, QtWidgets.QHeaderView.Stretch)
                header.setSectionResizeMode(1, QtWidgets.QHeaderView.Stretch)

                for i, label in enumerate(env_vars):
                    self.table.setItem(i, 0, QTableWidgetItem(label))
                    self.table.setItem(i, 1, QTableWidgetItem(env_vars[label]))

            else:
                self.table = QTableWidget(0, 2)
            self.table.setHorizontalHeaderLabels(["Variable", "Value"])
            self.table.setVerticalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOff)
            self.table.setHorizontalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOff)

            self.table.setMaximumSize(self._getQTableWidgetSize())
            self.form.addRow(QLabel("Environment Variables"), self.table)

            self.add_button = QPushButton("Add Variable")
            self.add_button.clicked.connect(self.add_variable)

            self.delete_button = QPushButton("Delete Variable")
            self.delete_button.clicked.connect(lambda: self.table.removeRow(self.table.currentRow()))

            self.form.addRow(self.add_button)
            self.form.addRow(self.delete_button)

        self.update_button = QPushButton("Update Settings")
        self.update_button.clicked.connect(self.update_config)
        self.form.addRow(self.update_button)

        self.setLayout(self.form)

    def _getQTableWidgetSize(self):
        w = self.table.verticalHeader().width() + 15
        for i in range(self.table.columnCount()):
            w += self.table.columnWidth(i)  # seems to include gridline (on my machine)
        h = self.table.horizontalHeader().height() + 15
        for i in range(self.table.rowCount() + 1):
            h += self.table.rowHeight(i)
            if h < 50:
                h = 75
        return QtCore.QSize(w, h)

    def add_Row(self, info_text: str, type_of_input: str, lgd_name: str, flags: [] = None):
        if flags is None:
            flags = []
        if os.name == "nt" and "wine" in flags:
            return
        field = None
        if not self.app_name in self.config.sections():
            self.config[self.app_name] = {}

        if not lgd_name in self.config[self.app_name]:
            self.config[self.app_name][lgd_name] = ""

        if type_of_input == "QLineEdit":
            field = QLineEdit(self.config[self.app_name][lgd_name])
            field.setPlaceholderText("Default")
            if "only_int" in flags:
                field.setValidator(QIntValidator())
            if "path" in flags:
                if lgd_name == "wine_prefix":
                    field = PathEdit(text=self.config[self.app_name][lgd_name], type_of_file=QFileDialog.DirectoryOnly,
                                     infotext="Select Wine Prefix")
                if lgd_name == "install_dir":
                    field = PathEdit(text=self.config[self.app_name][lgd_name], type_of_file=QFileDialog.DirectoryOnly,
                                     infotext="Select Default installation directory")
        elif type_of_input == "QComboBox":
            combo_list = []
            if "binary" in flags:
                combo_list = ["unset", "true", "false"]
            else:
                combo_list = flags
            field = QComboBox()
            field.addItems(combo_list)

            if self.config[self.app_name][lgd_name] == "":
                field.setCurrentIndex(0)
            elif self.config[self.app_name][lgd_name] == "true":
                field.setCurrentIndex(1)
            elif not self.config[self.app_name][lgd_name] == "false":
                field.setCurrentIndex(2)

        if not field:
            return

        self.form.addRow(QLabel(info_text), field)
        self.rows.append((field, type_of_input, lgd_name))
        # pprint(self.rows)

    def add_variable(self):
        self.table.insertRow(self.table.rowCount())
        self.table.setItem(self.table.rowCount(), 0, QTableWidgetItem(""))
        self.table.setItem(self.table.rowCount(), 1, QTableWidgetItem(""))
        self.table.setMaximumSize(self._getQTableWidgetSize())

    def update_config(self):
        config = {self.app_name: {}}
        for setting in self.rows:
            field, type_of_input, lgd_name = setting
            if type_of_input == "QLineEdit":
                if field.text() != "":
                    config[self.app_name][lgd_name] = field.text()
            elif type_of_input == "QComboBox":
                # print(type(field.currentText()), field.currentText())
                if field.currentText() == "true":
                    config[self.app_name][lgd_name] = "true"
                elif field.currentText() == "false":
                    config[self.app_name][lgd_name] = "false"
                elif field.currentText == "unset":
                    pass
                else:
                    pass
                    # config[self.app_name][lgd_name] = field.currentText()
        # Environment Variables
        if self.has_table:
            if self.table.rowCount() > 0:
                config[f"{self.app_name}.env"] = {}
                for row in range(self.table.rowCount()):
                    config[f"{self.app_name}.env"][self.table.item(row, 0).text()] = self.table.item(row, 1).text()
            else:
                config[f"{self.app_name}.env"] = {}
        logger.info(config)

        config_path = os.path.expanduser("~") + "/.config/legendary/"
        lgd_config = configparser.ConfigParser()
        lgd_config.optionxform = str

        lgd_config.read(config_path + "config.ini")

        logger.info(lgd_config.__dict__)
        if config[self.app_name] == {}:
            config.pop(self.app_name)
            lgd_config.remove_section(self.app_name)
        else:
            lgd_config[self.app_name] = config[self.app_name]

        if self.has_table:
            if config[self.app_name + ".env"] == {}:
                config.pop(self.app_name + ".env")
                lgd_config.remove_section(self.app_name + ".env")
            else:
                lgd_config[self.app_name + ".env"] = config[f"{self.app_name}.env"]
        # legendaryConfig.set_config()
        logger.info(f"Game: {self.app_name}/   {lgd_config.__dict__}")
        legendaryConfig.set_config(lgd_config)



    def use_proton_template(self):
        for i in self.rows:
            field, type_of_input, lgd_name = i

            if lgd_name == "wrapper":
                for file in os.listdir(os.path.expanduser("~/.steam/steam/steamapps/common")):
                    if file.startswith("Proton"):
                        protonpath = os.path.expanduser("~/.steam/steam/steamapps/common/" + file)
                        break
                else:
                    logger.error("No Proton found")
                    QMessageBox.Warning("Error", "No Proton was found")
                    return
                field.setText(os.path.join(protonpath, "proton") + " run")
            elif lgd_name == "no_wine":
                field.setCurrentIndex(1)

        self.table.insertRow(self.table.rowCount())

        self.table.setItem(self.table.rowCount() - 1, 0, QTableWidgetItem("STEAM_COMPAT_DATA_PATH"))
        self.table.setItem(self.table.rowCount() - 1, 1, QTableWidgetItem(os.path.expanduser("~/.proton")))



import time
from logging import getLogger

from PyQt5.QtCore import QThread, QProcess, pyqtSignal
from PyQt5.QtWidgets import QVBoxLayout, QMessageBox, QProgressBar, QPushButton, QLabel, QWidget, QHBoxLayout
from legendary.core import LegendaryCore
from legendary.models.game import InstalledGame

logger = getLogger("Updates")


class UpdateThread(QThread):
    finish = pyqtSignal(str)

    def __init__(self, dlm, core: LegendaryCore, igame):
        super(UpdateThread, self).__init__()
        self.dlm = dlm
        self.core = core
        self.igame = igame

    def run(self):
        try:
            start_time = time.time()
            self.dlm.start()
            self.dlm.join()
        except:
            self.finish.emit("Fail")

        else:
            end_time = time.time()
            self.core.install_game(self.igame)

            old_igame = self.core.get_installed_game(self.igame.app_name)
            print(old_igame.__dict__)
            """
            if old_igame and old_igame.install_tags != self.igame.install_tags:
                old_igame.install_tags = self.igame.install_tags
                logger.info('Deleting now untagged files.')
                self.core.uninstall_tag(old_igame)
                self.core.install_game(old_igame)
"""
            self.finish.emit(str(start_time - end_time))


class UpdateWidget(QWidget):
    installed = pyqtSignal()

    def __init__(self, game: InstalledGame, core: LegendaryCore):
        super(UpdateWidget, self).__init__()
        self.updating = False
        self.game = game
        self.core = core
        self.layout = QVBoxLayout()
        self.childlayout = QHBoxLayout()
        self.label = QLabel("Update available for " + self.game.title)
        self.button = QPushButton("Update")
        self.button.clicked.connect(self.update_game2)

        self.childlayout.addWidget(self.label)
        self.childlayout.addWidget(self.button)
        self.layout.addLayout(self.childlayout)
        self.progress_bar = QProgressBar()
        self.progress_bar.setMaximum(100)
        self.layout.addWidget(self.progress_bar)

        self.setLayout(self.layout)

    def update_game2(self):
        if not self.updating:
            logger.info("Update " + self.game.title)
            game = self.core.get_game(self.game.app_name)
            logger.info("Prepare Download")
            dlm, analysis, igame = self.core.prepare_download(game=game, base_game=game)

            if not analysis.dl_size:
                logger.info("Game is up to date, sorry")

            logger.info(f"Install size: {round(analysis.install_size / (1024 ** 3), 2)} GB")
            logger.info(f"Download size: {round(analysis.dl_size / (1024 ** 3), 2)} GB")
            installed_game = self.core.get_installed_game(self.game.app_name)
            res = self.core.check_installation_conditions(analysis, installed_game,
                                                          self.core.get_game(self.game.app_name), True)
            if res.failures:
                logger.error("Fail")
            if res.warnings:
                for warn in sorted(res.warnings):
                    logger.warning(warn)

            self.update_thread = UpdateThread(dlm, core=self.core, igame=igame)
            self.update_thread.finish.connect(self.finished)
            self.update_thread.start()
            self.button.setDisabled(True)
            # TODO Wird 1000% nicht funktionieren

        else:
            logger.info("Terminate process")
            self.thread.kill()
            self.button.setText("Update")
            self.updating = False

    def update_game(self):
        self.proc = QProcess()
        self.proc.start("legendary", ["-y", "update", self.game.app_name])

    def finished(self, text: str):
        if text == "Fail":
            QMessageBox.warning(self, "Update Failed", "Update failed")
        else:
            QMessageBox.information(self, "Update finished", f"Update finished in {time} ")
        self.setVisible(False)
        self.installed.emit()

    def start(self):
        self.button.setText("Cancel")
        self.updating = True

    def get_update_info(self):
        shit, infos, game = self.core.prepare_download(self.core.get_game(self.game.app_name),
                                                       self.core.get_game(self.game.app_name))
        logger.info(f"Download size: {infos.dl_size}")

    def dataReady(self):
        bytes = self.thread.readAllStandardOutput()
        byte_list = bytes.split('\n')
        data = []
        for i in byte_list:
            data.append(byte_list)

        text = data[0].decode()

        print(data)


class UpdateTab(QWidget):

    def __init__(self, core: LegendaryCore):
        super(UpdateTab, self).__init__()
        self.core = core
        self.layout = QVBoxLayout()
        update_games = self.get_updates()

        for game in update_games:
            widget = UpdateWidget(game, core)
            widget.installed.connect(lambda: self.__init__(self.core))
            self.layout.addWidget(widget)
        if len(update_games) == 0:
            self.layout.addWidget(QLabel("No updates available"))
        self.layout.addStretch(1)
        self.setLayout(self.layout)

    def get_updates(self):
        self.core.get_assets(True)
        update_games = []
        games = sorted(self.core.get_installed_list(), key=lambda x: x.title)
        versions = {}
        for game in games:
            try:
                versions[game.app_name] = self.core.get_asset(game.app_name).build_version
            except:
                logger.error(f"Metadata for {game.title} is missing")

            if versions[game.app_name] != game.version:
                update_games.append(game)
        return update_games



import os

from PyQt5.QtWidgets import *

from Rare.ext.SettingsForm import SettingsForm
from Rare.utils.constants import game_settings


class GameSettingsDialog(QDialog):
    action: str = ""

    def __init__(self, game, parent):
        super(GameSettingsDialog, self).__init__(parent=parent)
        self.game = game
        self.layout = QVBoxLayout()
        self.layout.addWidget(QLabel("<h1>Settings</h1>"))

        self.uninstall_button = QPushButton("Uninstall Game")

        self.settings = SettingsForm(self.game.app_name, game_settings, table=True)

        self.uninstall_button.clicked.connect(self.uninstall)
        self.exit_button = QPushButton("Exit")
        self.exit_button.clicked.connect(self.close)

        self.layout.addWidget(self.settings)

        if os.name != 'nt':
            self.use_proton = QPushButton("Use Proton")
            self.use_proton.clicked.connect(self.settings.use_proton_template)
            self.layout.addWidget(self.use_proton)
        self.layout.addWidget(self.uninstall_button)
        self.layout.addWidget(self.exit_button)

        self.setLayout(self.layout)

    def get_settings(self):
        self.exec_()
        return self.action

    def uninstall(self):
        msg = QMessageBox()
        msg.setIcon(QMessageBox.Warning)
        msg.setText(f"Do you really want to delete {self.game.title}")
        msg.setWindowTitle("Uninstall Game")
        msg.setStandardButtons(QMessageBox.Ok | QMessageBox.Cancel)
        ret = msg.exec_()

        if ret == QMessageBox.Ok:
            self.action = "uninstall"
            self.close()
            return
        else:
            self.action = ""
            self.close()

    def exit_settings(self):
        self.close()



import setuptools

from Rare import __version__ as version

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="Rare",
    version=version,
    author="Dummerle",
    author_email="lennardbaeumer@gmail.com",
    license="GPL-3",
    long_description=long_description,
    long_description_content_type="text/markdown",
    include_package_data=True,
    url="https://github.com/Dummerle/Rare",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.8",
        'License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)',
        "Operating System :: OS Independent"
    ],
    python_requires=">=3.8",
    entry_points=dict(console_scripts=["rare=Rare.Main:main"]),
    install_requires=[
        "legendary-gl",
        "requests<3.0",
        "pillow",
        "pyqtwebengine",
        "setuptools",
        "wheel"
    ]
)



from Rare.Tabs.Browser import BrowserTab
from Rare.Tabs.GamesInstalled.InstalledList import GameListInstalled
from Rare.Tabs.GamesUninstalled.GameListUninstalled import GameListUninstalled
from Rare.Tabs.Settings.SettingsTab import SettingsTab
from Rare.Tabs.Update import UpdateTab



import os
from logging import getLogger

from PyQt5.QtCore import QProcess, pyqtSignal
from PyQt5.QtGui import QPixmap
from PyQt5.QtWidgets import QWidget, QHBoxLayout, QLabel, QPushButton, QStyle, QVBoxLayout
from legendary.core import LegendaryCore
from legendary.models.game import InstalledGame

from Rare.Tabs.GamesInstalled.GameSettingsDialog import GameSettingsDialog
from Rare.utils import legendaryUtils
from Rare.utils.RareConfig import IMAGE_DIR

logger = getLogger("GameWidget")


class GameWidget(QWidget):
    proc: QProcess
    signal = pyqtSignal(str)

    # TODO Repair
    def __init__(self, game: InstalledGame, core: LegendaryCore):
        super(GameWidget, self).__init__()
        self.core = core
        self.game = game
        self.dev = core.get_game(self.game.app_name).metadata["developer"]
        self.title = game.title
        self.app_name = game.app_name
        self.version = game.version
        self.size = game.install_size
        self.launch_params = game.launch_parameters
        # self.dev =
        self.game_running = False
        self.layout = QHBoxLayout()
        if os.path.exists(f"{IMAGE_DIR}/{game.app_name}/FinalArt.png"):
            pixmap = QPixmap(f"{IMAGE_DIR}/{game.app_name}/FinalArt.png")
        elif os.path.exists(f"{IMAGE_DIR}/{game.app_name}/DieselGameBoxTall.png"):
            pixmap = QPixmap(f"{IMAGE_DIR}/{game.app_name}/DieselGameBoxTall.png")
        elif os.path.exists(f"{IMAGE_DIR}/{game.app_name}/DieselGameBoxLogo.png"):
            pixmap = QPixmap(f"{IMAGE_DIR}/{game.app_name}/DieselGameBoxLogo.png")
        else:
            logger.warning(f"No Image found: {self.game.title}")
            pixmap = None
        if pixmap:
            pixmap = pixmap.scaled(180, 240)
            self.image = QLabel()
            self.image.setPixmap(pixmap)
            self.layout.addWidget(self.image)

        ##Layout on the right
        self.childLayout = QVBoxLayout()

        play_icon = self.style().standardIcon(getattr(QStyle, 'SP_MediaPlay'))
        settings_icon = self.style().standardIcon(getattr(QStyle, 'SP_DirIcon'))
        self.title_widget = QLabel(f"<h1>{self.title}</h1>")
        self.app_name_label = QLabel(self.app_name)
        self.launch_button = QPushButton(play_icon, "Launch")
        self.launch_button.setObjectName("launch_game_button")
        self.launch_button.setFixedWidth(120)
        '''
        self.launch_button.setStyleSheet("""
            QPushButton{
                background-color: blue;
                color: white;
            }
        """)
        '''
        self.launch_button.clicked.connect(self.launch)
        if os.name != "nt":
            self.wine_rating = QLabel("Wine Rating: " + self.get_rating())
        self.developer_label = QLabel("Dev: " + self.dev)
        self.version_label = QLabel("Version: " + str(self.version))
        self.size_label = QLabel(f"Installed size: {round(self.size / (1024 ** 3), 2)} GB")
        self.settings_button = QPushButton(settings_icon, " Settings (Icon TODO)")
        self.settings_button.setFixedWidth(200)
        self.settings_button.clicked.connect(self.settings)

        self.childLayout.addWidget(self.title_widget)
        self.childLayout.addWidget(self.launch_button)
        self.childLayout.addWidget(self.app_name_label)
        self.childLayout.addWidget(self.developer_label)
        if os.name != "nt":
            self.childLayout.addWidget(self.wine_rating)
        self.childLayout.addWidget(self.version_label)
        self.childLayout.addWidget(self.size_label)
        self.childLayout.addWidget(self.settings_button)

        self.childLayout.addStretch(1)
        self.layout.addLayout(self.childLayout)
        self.layout.addStretch(1)
        self.setLayout(self.layout)

    def launch(self):
        print("Launch")
        if not self.game_running:
            logger.info(f"launching {self.title}")
            self.proc = legendaryUtils.launch_game(self.app_name, self.core)
            if not self.proc:
                print("Fail")
                return

            self.proc.finished.connect(self.finished)
            self.launch_button.setText("Kill")
            self.game_running = True

        else:
            self.kill()

    def finished(self, exit_code):
        self.launch_button.setText("Launch")
        logger.info(f"Game {self.title} finished with exit code {exit_code}")
        self.game_running = False

    def kill(self):
        self.proc.terminate()
        self.launch_button.setText("Launch")
        self.game_running = False
        logger.info("Killing Game")

    def get_rating(self) -> str:
        return "gold"  # TODO

    def settings(self):
        settings_dialog = GameSettingsDialog(self.game, self)
        action = settings_dialog.get_settings()
        del settings_dialog
        if action == "uninstall":
            legendaryUtils.uninstall(self.app_name, self.core)
            self.signal.emit(self.app_name)



import os
import shutil
from logging import getLogger

from PyQt5.QtWidgets import QGroupBox, QComboBox, QFormLayout, QLabel, QPushButton, QFileDialog

from Rare import style_path
from Rare.ext.QtExtensions import PathEdit
from Rare.utils import RareConfig

logger = getLogger("Rare Settings")


class RareSettingsForm(QGroupBox):
    def __init__(self):
        super(RareSettingsForm, self).__init__("Rare Settings")

        self.rare_form = QFormLayout()
        self.style_combo_box = QComboBox()
        self.style_combo_box.addItems(["Light", "Dark"])
        if RareConfig.THEME == "default":
            self.style_combo_box.setCurrentIndex(1)

        self.rare_form.addRow(QLabel("Style"), self.style_combo_box)

        self.image_dir_edit = PathEdit(RareConfig.IMAGE_DIR, QFileDialog.Directory, "Choose Folder")
        self.image_dir_edit.setPlaceholderText("Image directory")
        # self.image_dir_edit.setText(RareConfig.IMAGE_DIR)

        self.rare_form.addRow(QLabel("Image Directory"), self.image_dir_edit)
        self.submit_button = QPushButton("Update Rare Settings")
        self.submit_button.clicked.connect(self.update_rare_settings)
        self.rare_form.addRow(self.submit_button)
        self.setLayout(self.rare_form)

    def update_rare_settings(self):
        logger.info("Update Rare settings")
        config = {"Rare": {}}
        if self.style_combo_box.currentIndex() == 1:
            self.parent().parent().parent().parent().parent().setStyleSheet(open(style_path).read())
            config["Rare"]["theme"] = "default"
        else:
            self.parent().parent().parent().setStyleSheet("")
            config["Rare"]["theme"] = "light"
        config["Rare"]["IMAGE_DIR"] = self.image_dir_edit.text()

        if self.image_dir_edit.text() != RareConfig.IMAGE_DIR:
            shutil.move(RareConfig.IMAGE_DIR, self.image_dir_edit.text())
        RareConfig.set_config(config)
